#!/usr/bin/env python
######################################################################
## Author: Carl Schaefer, Smithsonian Institution Archives 
######################################################################

import sys
import os
import re
import argparse
sys.path.append(os.path.join(os.path.dirname(os.path.realpath(__file__)),
    'lib'))
import dm_common as dmc
from dm_common import EOL
import mbox2xml
import folder_data as fld

XML_WRAP  = True
NO_CHUNK  = 0
NO_LIMIT  = 0
NO_LEVELS = 0
LEVELS    = 1
LOG_NAME  = 'dm_xml.log.txt'
EOL       = os.linesep

fd_list   = []

######################################################################
def LogVariables (logf):
  logf.write('########## SETTINGS ##########' + EOL)
  logf.write('account: ' + account_name + EOL)
  logf.write('account directory: ' + account_directory + EOL)
#  if max_internal == dmc.ALLOCATE_BY_DISPOSITION:
  if max_internal == NO_LIMIT:
    logf.write('external content: all attachments' + EOL)
  else:
    logf.write('external content: all attachments > ' + \
      str(max_internal) + ' bytes' +EOL)
  if chunksize:
    logf.write('chunk size: ' + str(chunksize) + EOL)
  if folder_name:
    logf.write('folder: ' + folder_name + EOL)
  logf.write('external storage subdirectory levels: ' + \
      str(levels) + EOL)
  logf.write('xml-wrap externally stored content: ' + \
      str(xml_wrap) + EOL)

######################################################################
def CheckAccountDirectory (account_name, account_directory, fd_list):
  if not os.path.exists(account_directory):
    print 'account directory ' + account_directory + ' does not exist'
    return False
  if not os.path.isdir(account_directory):
    print account_directory +  ' is not a directory'
    return False
  fld.FindMboxFiles(account_name, account_directory, fd_list,
      account_directory)
  return True

########################################################################
def check_for_dup_folder_names (fd_list):
  fld_names = {}
  for fd in fd_list:
    if fd.folder_name not in fld_names.keys():
      fld_names[fd.folder_name] = 1
    else:
      fld_names[fd.folder_name] = fld_names[fd.folder_name] + 1
  dups = []
  for folder_name in fld_names.keys():
    if fld_names[folder_name] > 1:
      dups.append(folder_name)
  return dups

######################################################################
def CheckFolderList (fd_list, folder):
  if len(fd_list) == 0:
    print 'There are no .mbox files located under the account directory'
    return False
  dups = check_for_dup_folder_names(fd_list)
  if len(dups) > 0:
    msg = 'Folder names must be unique in an account. ' + \
        'These folder names are used more than one time in ' + \
        'the account directory:' + EOL + EOL.join(dups)
    print msg
    return False
  elif folder and not fld.FindFolderName(fd_list, folder):
    print 'File ' + folder + '.mbox cannot be found under the account directory'
    return False
  return True

######################################################################
def GetArgs():

  global account_directory
  global account_name
  global folder_name
  global folder_path
  global chunksize
  global levels
  global max_internal
  global xml_wrap

  account_directory = None
  account_name      = None
  folder_path       = None
  chunksize         = NO_CHUNK
  levels            = LEVELS
#  max_internal      = dmc.ALLOCATE_BY_DISPOSITION
  max_internal      = NO_LIMIT
  xml_wrap          = XML_WRAP

  parser = argparse.ArgumentParser(description='Convert mbox into XML.')
  parser.add_argument('--account', '-a', dest='account_name', required=True,
      help='email account name')
  parser.add_argument('--directory', '-d', dest='account_directory', required=True,
      help='directory to hold all files for this account')
  parser.add_argument('--folder', '-f', dest='folder_name',
      help='folder name (generate XML only for this one folder)')
  parser.add_argument('--max_internal', '-m', dest='max_internal',
      type=int, default=NO_LIMIT,
      help='maximum size in bytes for an internally-stored attachment, default = no limit')
  parser.add_argument('--chunk', '-c', dest='chunk', type=int,
      default=NO_CHUNK,
      help='number of messages to put in one output XML file, '+ \
      'default = no limit')
  parser.add_argument('--no_subdirectories', '-n', dest='no_subdirectories', action='store_true',
      help='do NOT make subdirectories to hold external content' + \
      '(default = make subdirectories)')

  args      = parser.parse_args()
  argdict   = vars(args)
  if 'max_internal' in argdict.keys():
    max_internal = int(argdict['max_internal'])
  if argdict['no_subdirectories']:
    levels = NO_LEVELS
  account_name      = argdict['account_name'].strip()
  account_directory = os.path.normpath(
      os.path.abspath(argdict['account_directory'].strip()))
  folder_name = argdict['folder_name']
  if folder_name:
    folder_name = folder_name.strip()
  if 'chunk' in argdict.keys():
    chunksize = argdict['chunk']

  if not CheckAccountDirectory(account_name, account_directory, fd_list):
    sys.exit(0)
  if not CheckFolderList(fd_list, folder_name):
    sys.exit(0)

######################################################################
def Convert ():  
  logf = open(os.path.join(account_directory, LOG_NAME), 'w')
  LogVariables(logf)
  mx = mbox2xml.Mbox2Xml(
      logf,
      max_internal,
      levels,
      chunksize,
      xml_wrap)
  if folder_name:
    for fd in fd_list:
      if folder_name == fd.folder_name:
        mx.walk_folder(fd)
  else:
    mx.walk_account(fd_list)
  mx.xc.finish_account()
  mx.finish_output_file()
  mx.write_log()
  del mx
  logf.close()

######################################################################
GetArgs()
Convert()
